import { Injectable } from '@angular/core';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import { of } from 'rxjs';
import { catchError, exhaustMap, map, tap } from 'rxjs/operators';

import { authSuccess, authFailure } from './actions';
import { initSettings } from '../init';

import {
  QueryProtocolService,
  AuthRequest
} from '@ucap-webmessenger/protocol-query';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { LoginInfo, KEY_LOGIN_INFO } from '@app/types';

@Injectable()
export class Effects {
  initSettings$ = createEffect(() =>
    this.actions$.pipe(
      ofType(initSettings),
      map(() => {
        const loginInfo = this.sessionStorageService.get<LoginInfo>(
          KEY_LOGIN_INFO
        );
        return {
          deviceType: loginInfo.deviceType
        } as AuthRequest;
      }),
      exhaustMap(req =>
        this.queryProtocolService.auth(req).pipe(
          map(res => authSuccess({ res })),
          catchError(error => of(authFailure({ error })))
        )
      )
    )
  );

  constructor(
    private actions$: Actions,
    private sessionStorageService: SessionStorageService,
    private queryProtocolService: QueryProtocolService
  ) {}
}
