import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  ProtocolDecoder,
  ProtocolMessage,
  PacketBodyValue,
  BodyStringDivider,
  ProtocolStream
} from '@ucap-webmessenger/protocol';
import { DeptType } from '../types/dept.type';

export interface DeptRequest extends ProtocolRequest {
  // DivCD(s)
  divCd: string;
  // 기관코드(s)
  companyCode: string;
}

export interface DeptInfo {
  // 부서SEQ
  seq: number;
  // 부서명
  name: string;
  // 기관코드
  companyCode: string;
  // 부서타입
  type: DeptType;
  // 본부SEQ
  rootSeq: number;
  // 상위부서SEQ
  parentSeq: number;
  // 조회순서
  order: string;
  // 조회레벨
  depth: number;
  // ActiveYN
  isActive: boolean;
  // 부서명(영어)
  nameEn: string;
  // 부서명(중국어)
  nameCn: string;
}
export interface DeptData extends ProtocolStream {
  // DivCD(s)
  divCd: string;
  // {부서정보-S}...
  departmentInfos: DeptInfo[];
}
export interface DeptResponse extends ProtocolResponse {
  // DivCD(s)
  divCd: string;
}

export const encodeDept: ProtocolEncoder<DeptRequest> = (req: DeptRequest) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({ type: PacketBodyValue.String, value: req.divCd });
  bodyList.push({ type: PacketBodyValue.String, value: req.companyCode });
  return bodyList;
};

export const decodeDeptData: ProtocolDecoder<DeptData> = (
  message: ProtocolMessage
) => {
  const departmentInfos: DeptInfo[] = [];
  message.bodyList.slice(1).forEach(deptInfo => {
    const info = deptInfo.split(BodyStringDivider);
    let i = 0;
    departmentInfos.push({
      seq: info[i],
      name: info[i++],
      companyCode: info[i++],
      type: info[i++] as DeptType,
      rootSeq: info[i++],
      parentSeq: info[i++],
      order: info[i++],
      depth: info[i++],
      isActive: info[i++] === 'Y' ? true : false,
      nameEn: info[i++],
      nameCn: info[i++]
    });
  });
  return {
    divCd: message.bodyList[0],
    departmentInfos
  } as DeptData;
};

export const decodeDept: ProtocolDecoder<DeptResponse> = (
  message: ProtocolMessage
) => {
  return { divCd: message.bodyList[0] } as DeptResponse;
};
